@@ -1523,11 +1523,12 @@ pango_attr_list_insert_before (PangoAttrList  *list,
  **/
 void
 pango_attr_list_change (PangoAttrList  *list,
-			PangoAttribute *attr)
+                        PangoAttribute *attr)
 {
   guint i, p;
   guint start_index = attr->start_index;
   guint end_index = attr->end_index;
+  gboolean inserted;
 
   g_return_if_fail (list != NULL);
 
@@ -1543,6 +1544,7 @@ pango_attr_list_change (PangoAttrList  *list,
       return;
     }
 
+  inserted = FALSE;
   for (i = 0, p = list->attributes->len; i < p; i++)
     {
       PangoAttribute *tmp_attr = g_ptr_array_index (list->attributes, i);
@@ -1550,6 +1552,7 @@ pango_attr_list_change (PangoAttrList  *list,
       if (tmp_attr->start_index > start_index)
         {
           g_ptr_array_insert (list->attributes, i, attr);
+          inserted = TRUE;
           break;
         }
 
@@ -1559,8 +1562,8 @@ pango_attr_list_change (PangoAttrList  *list,
       if (tmp_attr->end_index < start_index)
         continue; /* This attr does not overlap with the new one */
 
+      g_assert (tmp_attr->start_index <= start_index);
       g_assert (tmp_attr->end_index >= start_index);
-      g_assert (start_index <= tmp_attr->end_index);
 
       if (pango_attribute_equal (tmp_attr, attr))
         {
@@ -1579,21 +1582,22 @@ pango_attr_list_change (PangoAttrList  *list,
           pango_attribute_destroy (attr);
 
           attr = tmp_attr;
+          inserted = TRUE;
           break;
         }
       else
         {
           /* Split, truncate, or remove the old attribute
            */
-          if (tmp_attr->end_index > attr->end_index)
+          if (tmp_attr->end_index > end_index)
             {
               PangoAttribute *end_attr = pango_attribute_copy (tmp_attr);
 
-              end_attr->start_index = attr->end_index;
+              end_attr->start_index = end_index;
               pango_attr_list_insert (list, end_attr);
             }
 
-          if (tmp_attr->start_index == attr->start_index)
+          if (tmp_attr->start_index == start_index)
             {
               pango_attribute_destroy (tmp_attr);
               g_ptr_array_remove_index (list->attributes, i);
@@ -1601,12 +1605,12 @@ pango_attr_list_change (PangoAttrList  *list,
             }
           else
             {
-              tmp_attr->end_index = attr->start_index;
+              tmp_attr->end_index = start_index;
             }
         }
     }
 
-  if (i == p)
+  if (!inserted)
     {
       /* we didn't insert attr yet */
       pango_attr_list_insert (list, attr);
@@ -1693,40 +1697,41 @@ pango_attr_list_update (PangoAttrList *list,
 {
   guint i, p;
 
-  for (i = 0, p = list->attributes->len; i < p; i++)
-    {
-      PangoAttribute *attr = g_ptr_array_index (list->attributes, i);
+  if (list->attributes)
+    for (i = 0, p = list->attributes->len; i < p; i++)
+      {
+        PangoAttribute *attr = g_ptr_array_index (list->attributes, i);
 
-      if (attr->start_index >= pos &&
+        if (attr->start_index >= pos &&
           attr->end_index < pos + remove)
-        {
-          pango_attribute_destroy (attr);
-          g_ptr_array_remove_index (list->attributes, i);
-          i--; /* Look at this index again */
-          p--;
-          continue;
-        }
+          {
+            pango_attribute_destroy (attr);
+            g_ptr_array_remove_index (list->attributes, i);
+            i--; /* Look at this index again */
+            p--;
+            continue;
+          }
 
-      if (attr->start_index >= pos &&
-          attr->start_index < pos + remove)
-        {
-          attr->start_index = pos + add;
-        }
-      else if (attr->start_index >= pos + remove)
-        {
-          attr->start_index += add - remove;
-        }
+        if (attr->start_index >= pos &&
+            attr->start_index < pos + remove)
+          {
+            attr->start_index = pos + add;
+          }
+        else if (attr->start_index >= pos + remove)
+          {
+            attr->start_index += add - remove;
+          }
 
-      if (attr->end_index >= pos &&
-          attr->end_index < pos + remove)
-        {
-          attr->end_index = pos;
-        }
-      else if (attr->end_index >= pos + remove)
-        {
-          attr->end_index += add - remove;
-        }
-    }
+        if (attr->end_index >= pos &&
+            attr->end_index < pos + remove)
+          {
+            attr->end_index = pos;
+          }
+        else if (attr->end_index >= pos + remove)
+          {
+            attr->end_index += add - remove;
+          }
+      }
 }
 
 /**
@@ -1771,26 +1776,27 @@ pango_attr_list_splice (PangoAttrList *list,
  */
 #define CLAMP_ADD(a,b) (((a) + (b) < (a)) ? G_MAXUINT : (a) + (b))
 
-  for (i = 0, p = list->attributes->len; i < p; i++)
-    {
-      PangoAttribute *attr = g_ptr_array_index (list->attributes, i);;
+  if (list->attributes)
+    for (i = 0, p = list->attributes->len; i < p; i++)
+      {
+        PangoAttribute *attr = g_ptr_array_index (list->attributes, i);;
 
-      if (attr->start_index <= upos)
-	{
-	  if (attr->end_index > upos)
-	    attr->end_index = CLAMP_ADD (attr->end_index, ulen);
-	}
-      else
-	{
-	  /* This could result in a zero length attribute if it
-	   * gets squashed up against G_MAXUINT, but deleting such
-	   * an element could (in theory) suprise the caller, so
-	   * we don't delete it.
-	   */
-	  attr->start_index = CLAMP_ADD (attr->start_index, ulen);
-	  attr->end_index = CLAMP_ADD (attr->end_index, ulen);
-        }
-    }
+        if (attr->start_index <= upos)
+          {
+            if (attr->end_index > upos)
+              attr->end_index = CLAMP_ADD (attr->end_index, ulen);
+          }
+        else
+          {
+            /* This could result in a zero length attribute if it
+             * gets squashed up against G_MAXUINT, but deleting such
+             * an element could (in theory) suprise the caller, so
+             * we don't delete it.
+             */
+            attr->start_index = CLAMP_ADD (attr->start_index, ulen);
+            attr->end_index = CLAMP_ADD (attr->end_index, ulen);
+         }
+      }
 
   if (!other->attributes || other->attributes->len == 0)
     return;
