@@ -1006,6 +1006,8 @@ itemize_state_init (ItemizeState      *state,
   state->item = NULL;
 
   state->run_start = text + start_index;
+  state->changed = EMBEDDING_CHANGED | SCRIPT_CHANGED | LANG_CHANGED |
+                   FONT_CHANGED | WIDTH_CHANGED | EMOJI_CHANGED;
 
   /* First, apply the bidirectional algorithm to break
    * the text into directional runs.
@@ -1079,7 +1081,6 @@ itemize_state_init (ItemizeState      *state,
   state->cache = NULL;
   state->base_font = NULL;
 
-  state->changed = EMBEDDING_CHANGED | SCRIPT_CHANGED | LANG_CHANGED | FONT_CHANGED | WIDTH_CHANGED | EMOJI_CHANGED;
 }
 
 static gboolean
@@ -1565,7 +1566,7 @@ pango_itemize_with_base_dir (PangoContext      *context,
   g_return_val_if_fail (length >= 0, NULL);
   g_return_val_if_fail (length == 0 || text != NULL, NULL);
 
-  if (length == 0)
+  if (length == 0 || g_utf8_strlen (text, length) == 0)
     return NULL;
 
   itemize_state_init (&state, context, text, base_dir, start_index, length,
